#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import pickle

# import interfaces
from pyams_cache.interfaces import ICacheHandler
from pyams_notify.interfaces import CACHE_QUEUE_KEY, CACHE_CONFIGURATION_KEY
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_cache.cache import get_cache_handler
from pyramid.view import view_config


def filtered(notification, request):
    """Filter notification against current request"""
    return request.principal.id in notification['target'].get('principals', ())


@view_config(name='get-user-notifications.json', request_type=IPyAMSLayer, renderer='json', xhr=True)
class UserNotificationsView(object):
    """User notifications view"""

    def __init__(self, request):
        self.request = request
        self.context = request.context

    @property
    def cache_server(self):
        return self.request.registry.settings.get(CACHE_CONFIGURATION_KEY)

    def __call__(self):
        server = self.cache_server
        if server is not None:
            client = get_cache_handler(server, ICacheHandler)
            if client is None:
                return ()
            notifications = client.get(CACHE_QUEUE_KEY)
            if notifications is None:
                return ()
            else:
                return [n for n in reversed(list(filter(lambda x: filtered(x, self.request),
                                                        pickle.loads(notifications))))]
        else:
            return ()
